@@ -3,7 +3,7 @@
  * $Id: doxmlintf.h,v 1.29 2002/10/13 21:01:58 dimitri Exp $
  *
  *
- * Copyright (C) 1997-2004 by Dimitri van Heesch.
+ * Copyright (C) 1997-2005 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -33,6 +33,7 @@ class ICompound;
 class ISection;
 class INode;
 class IDocInternal;
+class IDocRoot;
 
 /*! \brief Read only interface to a string. 
  */
@@ -51,6 +52,11 @@ class IString
     virtual int length() const = 0;
 };
 
+/*! \brief Base interface for hyperlinked text
+ *
+ *  Depending on the result of kind() the interface is extended by
+ *  ILT_Text or ILT_Ref.
+ */
 class ILinkedText
 {
   public:
@@ -58,12 +64,16 @@ class ILinkedText
     virtual Kind kind() const = 0;
 };
 
+/*! \brief Plain text fragment.
+ */
 class ILT_Text : public ILinkedText
 {
   public:
     virtual const IString *text() const = 0;
 };
 
+/*! \brief Reference to an object.
+ */
 class ILT_Ref : public ILinkedText
 {
   public:
@@ -74,6 +84,8 @@ class ILT_Ref : public ILinkedText
     virtual const IString *text() const = 0;
 };
 
+/*! \brief Iterates over a list of ILinkedText fragments.
+ */
 class ILinkedTextIterator 
 {
   public:
@@ -85,6 +97,7 @@ class ILinkedTextIterator
     virtual void release() = 0;
 };
 
+/*! \brief Representation of a parameter of a function. */
 class IParam 
 {
   public:
@@ -94,6 +107,7 @@ class IParam
     virtual const IString * attrib() const = 0;
     virtual const IString * arraySpecifier() const = 0;
     virtual ILinkedTextIterator *defaultValue() const = 0;
+    virtual IDocRoot *briefDescription() const = 0;
 };
 
 class IParamIterator
@@ -171,7 +185,8 @@ class IDoc
       Anchor,             // 34 -> IDocAnchor
       Symbol,             // 35 -> IDocSymbol
       Internal,           // 36 -> IDocInternal
-      Root                // 37 -> IDocRoot
+      Root,               // 37 -> IDocRoot
+      ParameterItem       // 38 -> IDocParameterItem
     };
     virtual Kind kind() const = 0;
 };
@@ -238,15 +253,21 @@ class IDocParameterList : public IDoc
 {
   public:
     enum Types { Param, RetVal, Exception };
-    virtual Types listType() const = 0;
+    virtual Types sectType() const = 0;
     virtual IDocIterator *params() const = 0;
 };
 
+class IDocParameterItem : public IDoc
+{
+  public:
+    virtual IDocIterator *paramNames() const = 0;
+    virtual IDocPara *description() const = 0;
+};
+
 class IDocParameter : public IDoc
 {
   public:
     virtual const IString * name() const = 0;
-    virtual IDocPara *description() const = 0;
 };
 
 class IDocTitle : public IDoc
@@ -465,6 +486,7 @@ class IDocRoot : public IDoc
 {
   public:
     virtual IDocIterator *contents() const = 0; 
+    virtual IDocInternal *internal() const = 0; 
 };
 
 class IDocIterator 
@@ -562,14 +584,25 @@ class IMember
     virtual ILinkedTextIterator *type() const = 0;
     virtual const IString * typeString() const = 0;
     virtual const IString * name() const = 0;
+    virtual const IString * readAccessor() const = 0;
+    virtual const IString * writeAccessor() const = 0;
+    virtual const IString * definition() const = 0;
+    virtual const IString * argsstring() const = 0;
     virtual bool isConst() const = 0;
     virtual bool isVolatile() const = 0;
+    virtual bool isStatic() const = 0;
+    virtual bool isExplicit() const = 0;
+    virtual bool isInline() const = 0;
+    virtual bool isMutable() const = 0;
+    virtual bool isReadable() const = 0;
+    virtual bool isWritable() const = 0;
     virtual IParamIterator *parameters() const = 0;
     virtual IParamIterator *templateParameters() const = 0;
     virtual ILinkedTextIterator *initializer() const = 0;
     virtual ILinkedTextIterator *exceptions() const = 0;
     virtual IMemberReferenceIterator *references() const = 0; 
     virtual IMemberReferenceIterator *referencedBy() const = 0;
+    virtual const IString *bodyFile() const = 0;
     virtual int bodyStart() const = 0;
     virtual int bodyEnd() const = 0;
     virtual const IString * definitionFile() const = 0;
@@ -578,6 +611,7 @@ class IMember
     virtual IMemberReferenceIterator *reimplementedBy() const = 0;
     virtual IDocRoot *briefDescription() const = 0;
     virtual IDocRoot *detailedDescription() const = 0;
+    virtual IDocRoot *inbodyDescription() const = 0;
 };
 
 class IDefine : public IMember
@@ -636,19 +670,26 @@ class IEnumValue : public IMember
     virtual const IString * name() const = 0;
 };
 
-#if 0
-class IEnumValueIterator 
+/*! \brief Include relation
+ */
+class IInclude 
 {
   public:
-    virtual IEnumValue *toFirst() = 0;
-    virtual IEnumValue *toLast() = 0;
-    virtual IEnumValue *toNext() = 0;
-    virtual IEnumValue *toPrev() = 0;
-    virtual IEnumValue *current() const = 0;
+    virtual const IString * name() const = 0;
+    virtual const IString * refId() const = 0;
+    virtual bool isLocal() const = 0;
+};
+
+class IIncludeIterator
+{
+  public:
+    virtual IInclude *toFirst() = 0;
+    virtual IInclude *toLast() = 0;
+    virtual IInclude *toNext() = 0;
+    virtual IInclude *toPrev() = 0;
+    virtual IInclude *current() const = 0;
     virtual void release() = 0;
 };
-#endif
-
 
 class IMemberIterator 
 {
@@ -691,6 +732,7 @@ class ISection
       Signals,             //!< Qt Signals
       DCOPFuncs,           //!< KDE-DCOP interface functions
       Properties,          //!< IDL properties
+      Events,              //!< C# events
       PubStatFuncs,        //!< Public static member functions
       PubStatAttribs,      //!< Public static attributes
       ProTypes,            //!< Protected member typedefs
@@ -725,6 +767,11 @@ class ISection
     
     /*! Returns what kind of section this is */
     virtual SectionKind kind() const = 0;
+
+    /*! Returns the description attached to this section (for user defined
+     *  sections, also known as member groups).
+     */
+    virtual IDocRoot *description() const = 0;
     
     /*! Returns an iterator for the members of this section */
     virtual IMemberIterator *members() const = 0;
@@ -795,9 +842,10 @@ class ICompound
   public:
     /*! Represents the kind of compounds recognised by doxygen. */
     enum CompoundKind { Invalid=0,
-                        Class, Struct, Union, Interface, Exception,
-                        Namespace, File, Group, Page
+                        Class, Struct, Union, Interface, Protocol, Category,
+                        Exception, File, Namespace, Group, Page, Example, Dir
                       };
+
     /*! Returns the name of this compound */
     virtual const IString * name() const = 0;
 
@@ -868,6 +916,7 @@ class IRelatedCompound
     virtual ICompound *compound() const = 0;
     virtual Protection protection() const = 0;
     virtual Kind kind() const = 0;
+    virtual const IString *name() const = 0;
 
 };
 
@@ -895,6 +944,7 @@ class IClass : public ICompound
     virtual IParamIterator *templateParameters() const = 0;
     virtual const IString *locationFile() const = 0;
     virtual int locationLine() const = 0;
+    virtual const IString *locationBodyFile() const = 0;
     virtual int locationBodyStartLine() const = 0;
     virtual int locationBodyEndLine() const = 0;
 
@@ -936,6 +986,7 @@ class IInterface : public ICompound
     virtual IRelatedCompoundIterator *derivedCompounds() const = 0;
 };
 
+
 /*! \brief The interface to a Java/IDL exception in the object model. 
  */
 class IException : public ICompound
@@ -960,9 +1011,9 @@ class IFile : public ICompound
     virtual IDocProgramListing *source() const = 0;
     virtual ICompoundIterator *nestedCompounds() const = 0;
 
-    // file:
-    //  includes()
-    //  includedBy()
+    virtual IIncludeIterator *includes() const = 0;
+    virtual IIncludeIterator *includedBy() const = 0;
+
     //  ICompound *innerNamespaces()
     //  ICompoundIterator *innerClasses()
 };
